require "import"
import "android.widget.*"
import "com.androlua.*"
import "utf8"

local layout = {
  LinearLayout,
  orientation = "vertical",
  layout_width = "fill",
  layout_height = "fill",
  padding = "16dp",
  {EditText,
    id = "inputText",
    hint = "Digite seu texto",
    layout_width = "fill",
    layout_height = "0dp",
    layout_weight = 1,
    padding = "8dp",
    inputType = "textMultiLine"},
  {ScrollView,
    layout_width = "fill",
    layout_height = "wrap_content",
    {LinearLayout,
      orientation = "vertical",
      layout_width = "fill",
      layout_height = "wrap_content",
      {TextView,
        id = "characterCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "wordCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "lineCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "paragraphCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "sentenceCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "emojiCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "symbolCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"},
      {TextView,
        id = "spaceCount",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "8dp",
        textSize = "18sp"}}},
  {Button,
    id = "botaoLimpar",
    text = "Limpar tudo",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "8dp" },
  {Button,
    id = "botaoSair",
    text = "Sair",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "8dp" }}

local function atualizarContagens()
  local texto = inputText.text
  if texto then
    local linhas = {}
    for linha in texto:gmatch("([^\n]*)\n?") do
      table.insert(linhas, linha)
    end

    local palavras = {}
    for palavra in texto:gmatch("%S+") do
      table.insert(palavras, palavra)
    end

    local paragrafos = {}
    for paragrafo in texto:gmatch("([^\n\n]+)\n?\n?") do
      table.insert(paragrafos, paragrafo)
    end

    local frases = {}
    for frase in texto:gmatch("([^.!?]+[.!?])") do
      table.insert(frases, frase)
    end

    local emojis = 0
    for i = 1, #texto do
      local c = texto:sub(i,i)
      if utf8.codepoint(c) >= 128512 and utf8.codepoint(c) <= 128591 then
        emojis = emojis + 1
      end
    end

    local simbolos = 0
    for i = 1, #texto do
      local c = texto:sub(i,i)
      if not c:match("%a") and not c:match("%d") and not c:match("%s") and not c:match("%p") then
        simbolos = simbolos + 1
      end
    end

    local espacos = 0
    for i = 1, #texto do
      local c = texto:sub(i,i)
      if c:match("%s") then
        espacos = espacos + 1
      end
    end

    characterCount.text = "Número de caracteres: ".. #texto
    wordCount.text = "Número de palavras: ".. #palavras
    lineCount.text = "Número de linhas: ".. #linhas
    paragraphCount.text = "Número de parágrafos: ".. #paragrafos
    sentenceCount.text = "Número de frases: ".. #frases
    emojiCount.text = "Número de emojis: ".. emojis
    symbolCount.text = "Número de símbolos: ".. simbolos
    spaceCount.text = "Número de espaços: ".. espacos
  else
    characterCount.text = "Número de caracteres: 0"
    wordCount.text = "Número de palavras: 0"
    lineCount.text = "Número de linhas: 0"
    paragraphCount.text = "Número de parágrafos: 0"
    sentenceCount.text = "Número de frases: 0"
    emojiCount.text = "Número de emojis: 0"
    symbolCount.text = "Número de símbolos: 0"
    spaceCount.text = "Número de espaços: 0"
  end
end

local dialog = LuaDialog(this)
dialog.setTitle("Análise de Texto")
dialog.setView(loadlayout(layout))

inputText.addTextChangedListener {
  onTextChanged = function(texto)
    atualizarContagens()
  end}

botaoLimpar.onClick = function()
  inputText.text = ""
  atualizarContagens()
end

botaoSair.onClick = function()
  dialog.dismiss()
end

atualizarContagens()
dialog.show()